@@ -19,6 +19,7 @@
 import docutils
 import docutils.nodes
 from typing import NamedTuple, Any, List, Dict
+import copy
 
 try:
     from .uml import uml
@@ -36,6 +37,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
     _MACRO_FUNCTION_TABLE='macro_function_table'
     _SOURCE_FILE_DEPENDENCIES='source_file_dependencies'
     _SOURCE_FILE_TABLE='file_table'
+    _VARIABLE_TABLE='variable_table'
 
     class KeyEntry(NamedTuple):
         row_label: str
@@ -48,14 +50,16 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
         KeyEntry('Type name:', 'type-name', dict([(_TYPE_TABLE, 1)])),
         KeyEntry('Function name:', 'function-name', dict([(_FUNCTION_TABLE, 1)])),
         KeyEntry('Constants Group:', 'constants-group', dict([(_MACRO_CONSTANTS_TABLE, 1)])),
+        KeyEntry('Variables Group:', 'variables-group', dict([(_VARIABLE_TABLE, 1)])),
         KeyEntry('Identifier name:', 'identifier-name', dict([(_MACRO_FUNCTION_TABLE, 1)])),
         KeyEntry('Description:', 'description', dict([(_TYPE_TABLE, 2), (_FUNCTION_TABLE, 2), (_MACRO_FUNCTION_TABLE, 2)])),
-        KeyEntry('Declared in:', 'header', dict([(_TYPE_TABLE, 4), (_FUNCTION_TABLE, 4), (_MACRO_CONSTANTS_TABLE, 2), (_MACRO_FUNCTION_TABLE, 4)])),
+        KeyEntry('Declared in:', 'header', dict([(_TYPE_TABLE, 4), (_FUNCTION_TABLE, 4), (_MACRO_CONSTANTS_TABLE, 2),
+                                                 (_MACRO_FUNCTION_TABLE, 4),(_VARIABLE_TABLE, 2)])),
         KeyEntry('Constants:', 'constants', dict([(_MACRO_CONSTANTS_TABLE, 3), (_TYPE_TABLE, 5)])),
+        KeyEntry('Variables:', 'variables', dict([(_VARIABLE_TABLE, 3)])),
         KeyEntry('Kind:', 'kind', dict([(_TYPE_TABLE, 3)])),
         KeyEntry('Type:', 'type', dict([(_TYPE_TABLE, 5)])),
         KeyEntry('Elements:', 'elements', dict([(_TYPE_TABLE, 5)])),
-        #KeyEntry('Range:', 6, 'range', [_TYPE_TABLE]),
         KeyEntry('Syntax:', 'syntax', dict([(_FUNCTION_TABLE, 3), (_MACRO_FUNCTION_TABLE, 3)])),
         KeyEntry('May be called from ISR:', 'allowed-from-isr', dict([(_FUNCTION_TABLE, 5), (_MACRO_FUNCTION_TABLE, 5)])),
         KeyEntry('Reentrancy:', 'is-reentrant', dict([(_FUNCTION_TABLE, 6), (_MACRO_FUNCTION_TABLE, 6)])),
@@ -73,22 +77,24 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
         self._lines = text.splitlines()
         self._state = self._PASS
         self._verbose = verbose
-        self.rownum = -1
+        self._rownum = -1
         self._globals = {}
+        self._private_section = False
 
     def _verbose_print(self, *args, **kwargs):
         if self._verbose:
             print(*args, **kwargs)
 
     _SECTION_STATE_MAP = {
-        "Module Interface Types" : _TYPE_TABLE,
-        "Module Interface Functions" : _FUNCTION_TABLE,
-        "Module Interface Constants" : _MACRO_CONSTANTS_TABLE,
-        "Module Interface Function-like Macros" : _MACRO_FUNCTION_TABLE,
+        "Types" : _TYPE_TABLE,
+        "Functions" : _FUNCTION_TABLE,
+        "Constants" : _MACRO_CONSTANTS_TABLE,
+        "Variables" : _VARIABLE_TABLE,
+        "Function-like Macros" : _MACRO_FUNCTION_TABLE,
+        "Source File Description" : _SOURCE_FILE_TABLE,
         "Source File Dependencies" : _SOURCE_FILE_DEPENDENCIES,
-        "Module Compile-time Configuration" : _MACRO_CONSTANTS_TABLE,
-        "Module Link-time Configuration" : _TYPE_TABLE,
-        "Source File Description" : _SOURCE_FILE_TABLE
+        "Compile-time Configuration" : _MACRO_CONSTANTS_TABLE,
+        "Link-time Configuration" : _TYPE_TABLE,
     }
 
     class HeaderAttribute(NamedTuple):
@@ -98,6 +104,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
     _HEADER_ATTRIBUTES = [
         HeaderAttribute('functions', []),
         HeaderAttribute('types', []),
+        HeaderAttribute('variables', []),
         HeaderAttribute('macro-constants', []),
         HeaderAttribute('macro-functions', []),
         HeaderAttribute('includes', []),
@@ -111,10 +118,13 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
             if isinstance(c, docutils.nodes.title):
                 self._verbose_print("Parsing section: " + c.astext())
                 current_section = c.astext()
-                if current_section in self._SECTION_STATE_MAP:
-                    self._state = self._SECTION_STATE_MAP[current_section]
-                    self._verbose_print("Parsing " + self._state.replace('_', ' '))
-                    self._elem_section = node
+                for s in self._SECTION_STATE_MAP:
+                    if s in current_section:
+                        self._state = self._SECTION_STATE_MAP[s]
+                        self._verbose_print("Parsing " + self._state.replace('_', ' '))
+                        self._elem_section = node
+                        self._private_section = ('Private' in current_section)
+                        break
                 break
 
     def depart_section(self,node: docutils.nodes.section) -> None:
@@ -125,15 +135,15 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
     def _add_header(self, header):
         self._headers.setdefault(header, {})
         for hattr in self._HEADER_ATTRIBUTES:
-            self._headers[header].setdefault(hattr.name, hattr.default)
+            self._headers[header].setdefault(hattr.name, copy.deepcopy(hattr.default))
         self._headers[header]['file-name'] = header
 
     def visit_table(self, node: docutils.nodes.table) -> None:
-        self.rownum = 0
+        self._rownum = 0
         self._elem_attributes = {}
 
     def depart_table(self, node):
-        self.rownum = -1
+        self._rownum = -1
         if self._state != self._PASS and 'header' in self._elem_attributes:
             header = self._elem_attributes['header']
             if header not in self._headers:
@@ -143,25 +153,33 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                 self._elem_attributes.setdefault('in-params', [])
                 self._elem_attributes.setdefault('out-params', [])
                 self._elem_attributes.setdefault('inout-params', [])
+                self._elem_attributes['private'] = self._private_section
                 self._headers[header]['functions'].append(self._elem_attributes)
             elif self._state == self._TYPE_TABLE:
                 #TODO check type attributes
+                self._elem_attributes['private'] = self._private_section
                 self._headers[header]['types'].append(self._elem_attributes)
             elif self._state == self._MACRO_CONSTANTS_TABLE:
+                self._elem_attributes['private'] = self._private_section
                 self._headers[header]['macro-constants'].append(self._elem_attributes)
             elif self._state == self._MACRO_FUNCTION_TABLE:
                 #TODO check function macro attributes
+                self._elem_attributes['private'] = self._private_section
                 self._elem_attributes.setdefault('in-params', [])
                 self._elem_attributes.setdefault('out-params', [])
                 self._elem_attributes.setdefault('inout-params', [])
                 self._headers[header]['macro-functions'].append(self._elem_attributes)
+            elif self._state == self._VARIABLE_TABLE:
+                #TODO check variable attributes
+                self._elem_attributes['private'] = self._private_section
+                self._headers[header]['variables'].append(self._elem_attributes)
         elif self._state == self._SOURCE_FILE_TABLE:
             pass
         elif self._state != self._PASS:
             raise RuntimeError('Invalid syntax: missing header in SW element specification')
 
     def visit_row(self, node):
-        self.rownum += 1
+        self._rownum += 1
 
     # Nested rows (e.g. function parameters) have the 1st column omitted by the grid table parser,
     # so we have to figure out the column number depending on the count of '|' before the paragraph text.
@@ -179,7 +197,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
         return descr
 
     def _add_type_attribute(self, colnum, node, content):
-        if self.rownum < 5:
+        if self._rownum < 5:
             self._assert_syntax(colnum == 2, node.line)
             self._assert_syntax(self._attr_to_add not in self._elem_attributes, node.line)
             self._elem_attributes[self._attr_to_add] = self._get_description(content) if self._attr_to_add == 'description' else content
@@ -210,7 +228,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                     self._elem_attributes[self._attr_to_add][-1]['description'] = self._get_description(content)
 
     def _add_function_attribute(self, colnum, node, content):
-        if self.rownum <= 6:
+        if self._rownum <= 6:
             self._assert_syntax(colnum == 2, node.line)
             self._assert_syntax(self._attr_to_add not in self._elem_attributes, node.line)
             if self._attr_to_add == 'description':
@@ -221,7 +239,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                 self._elem_attributes[self._attr_to_add] = (content == 'Reentrant') or 'Yes' in content or 'yes' in content
             else:
                 self._elem_attributes[self._attr_to_add] = content
-        elif self.rownum == 7:
+        elif self._rownum == 7:
             self._assert_syntax(colnum in [2, 3], node.line)
             if colnum == 2:
                 if content == 'None' or content == 'none':
@@ -238,7 +256,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                 self._elem_attributes[self._attr_to_add][-1]['description'] = self._get_description(content)
 
     def _add_macro_constants_attribute(self, colnum, node, content):
-        if self.rownum < 3:
+        if self._rownum < 3:
             self._assert_syntax(colnum == 2, node.line)
             self._assert_syntax(self._attr_to_add not in self._elem_attributes, node.line)
             self._elem_attributes[self._attr_to_add] = self._get_description(content) if self._attr_to_add == 'description' else content
@@ -252,8 +270,26 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
             else:
                 self._elem_attributes[self._attr_to_add][-1]['description'] = self._get_description(content)
 
+    def _add_variable_attribute(self, colnum, node, content):
+        # _attr_to_add = 'variables-group'
+        if self._rownum < 3:
+            self._assert_syntax(colnum == 2, node.line)
+            self._assert_syntax(self._attr_to_add not in self._elem_attributes, node.line)
+            self._elem_attributes[self._attr_to_add] = content.strip()
+        else:
+            self._assert_syntax(colnum in [2, 3], node.line)
+            self._elem_attributes.setdefault(self._attr_to_add, [])
+            if colnum == 2:
+                # Description or Syntax
+                subattr = content.lower().replace(':', '')
+                if subattr == 'description':
+                    self._elem_attributes[self._attr_to_add].append({})
+                self._subattr_to_add = subattr
+            else:
+                self._elem_attributes[self._attr_to_add][-1][self._subattr_to_add] = content
+
     def _add_macro_function_attribute(self, colnum, node, content):
-        if self.rownum < 7:
+        if self._rownum < 7:
             self._assert_syntax(colnum == 2, node.line)
             self._assert_syntax(self._attr_to_add not in self._elem_attributes, node.line)
             if self._attr_to_add == 'description':
@@ -264,7 +300,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                 self._elem_attributes[self._attr_to_add] = (content == 'Reentrant') or 'Yes' in content or 'yes' in content
             else:
                 self._elem_attributes[self._attr_to_add] = content
-        elif self.rownum == 7:
+        elif self._rownum == 7:
             self._assert_syntax(colnum in [2, 3], node.line)
             if colnum == 2:
                 if content == 'None' or content == 'none':
@@ -304,7 +340,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                         self._elem_attributes[self._attr_to_add][-1]['prepro-conditional'] = "#else"
 
     def _add_file_description(self, colnum, node, content):
-        if self.rownum == 1:
+        if self._rownum == 1:
             if colnum == 1:
                 self._assert_syntax("File" in content, node.line)
             elif colnum == 2:
@@ -328,7 +364,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
 
     def visit_paragraph(self, node: docutils.nodes.paragraph) -> None:
         content = node.astext()
-        if self.rownum > 0 and self._state != self._PASS:
+        if self._rownum > 0 and self._state != self._PASS:
             colnum = self.get_colnum(node, content)
 
             if self._state == self._SOURCE_FILE_TABLE:
@@ -339,7 +375,7 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                     if row_spec.row_label in content and self._state in row_spec.tables.keys():
                         expected_rownum = row_spec.tables[self._state]
                         if expected_rownum > -1:
-                            self._assert_syntax(self.rownum == expected_rownum, node.line)
+                            self._assert_syntax(self._rownum == expected_rownum, node.line)
                         self._attr_to_add = row_spec.attr_key
                         row_label_found = True
                         break
@@ -358,21 +394,21 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
     # code blocks are handled here
     def visit_literal_block(self, node: docutils.nodes.literal_block) -> None:
         content = node.astext()
-        if self.rownum > 0 and self._state != self._PASS:
+        if self._rownum > 0 and self._state != self._PASS:
             # literal_block node has no line
             #colnum = self.get_colnum(node, content)
 
             if self._state == self._TYPE_TABLE:
-                if self.rownum == 5:
+                if self._rownum == 5:
                     if self._attr_to_add == 'type':
                         self._elem_attributes[self._attr_to_add] = self._strip_code_block(content)
             elif self._state == self._FUNCTION_TABLE:
-                if self.rownum == 3:
+                if self._rownum == 3:
                     self._assert_syntax(self._attr_to_add not in self._elem_attributes, node.line)
                     self._elem_attributes[self._attr_to_add] = self._strip_code_block(content)
             elif self._state == self._MACRO_FUNCTION_TABLE:
                 if self._attr_to_add == 'syntax':
-                    if self.rownum == 3:
+                    if self._rownum == 3:
                         self._assert_syntax(self._attr_to_add not in self._elem_attributes, node.line)
                         self._elem_attributes[self._attr_to_add] = self._strip_code_block(content)
                 if self._attr_to_add == 'definition':
@@ -382,11 +418,15 @@ class RestProcessor(docutils.nodes.SparseNodeVisitor):
                     if 'condition' not in self._elem_attributes[self._attr_to_add][-1]:
                         self._elem_attributes[self._attr_to_add][-1]['condition'] = ''
                         self._elem_attributes[self._attr_to_add][-1]['prepro-conditional'] = ''
+            elif self._state == self._VARIABLE_TABLE:
+                self._elem_attributes[self._attr_to_add][-1][self._subattr_to_add] = content
 
 
     def visit_uml(self, node: uml) -> None:
         if self._state == self._SOURCE_FILE_DEPENDENCIES:
             depScanner = UmlDependencyScanner()
+            self._assert_syntax(hasattr(node, 'parse_tree'), node.line,
+                                        msg='File dependency diagram not found, or missing :restuml2code: directive')
             depScanner.visit_topdown(tree=node.parse_tree)
             for header in depScanner.header_deps:
                 if header not in self._headers:
